/***************************************************************************
 *            socket.c
 *
 *  Thu Jul  1 00:36:12 2004
 *  Copyright  2004  Michael Rasmussen
 *  mir@miras.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "socket.h"

/************************************************************/
/*															*/
/* initTcpServer: Create a TCP server						    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	port			: The port to listen on 0<port<65536	*/
/*	max				: No of concurrent connections			*/
/************************************************************/
/* Post conditions:											*/
/*	server.fd >= 0: The file descriptor for the server   	*/
/*	server.fd = -1: Error creating server					*/
/************************************************************/

Tcpsocket initTcpServer(int port, int max) {
//Tcpsocket server = (Tcpsocket) malloc(SOCKETSIZE);
Tcpsocket server;
int		i=1;

	server.fd = -1;
	bzero(&server.addr, SOCKETSIZE);
	server.addr.sin_family = AF_INET;
	server.addr.sin_port = htons(port);
	server.addr.sin_addr.s_addr = htonl(INADDR_ANY);
	
	if ((server.fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return server;
	
	// Set quarentine time for port number to 0
	setsockopt(server.fd, SOL_SOCKET, SO_REUSEADDR, (void *) &i, sizeof(int));
	
	if (bind(server.fd, (struct sockaddr *) &server.addr, sizeof(server.addr)) < 0) {
		close(server.fd);
		server.fd = -1;
		return server;
	}
	
	if (listen(server.fd, max) < 0) {
		close(server.fd);
		server.fd = -1;
		return server;
	}
	
	return server;
}

/************************************************************/
/*															*/
/* listenServer: Create a listening socket				    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	server			: A valid server socket					*/
/************************************************************/
/* Post conditions:											*/
/*	An open file descriptor > 0: Server is waiting for call	*/
/*	A file descriptor = 0: Error creating server			*/
/************************************************************/

int listenServer(Tcpsocket server) {
size_t s = sizeof(server.addr);
	
	return accept(server.fd, (struct sockaddr *) &server.addr, &s);
}

/************************************************************/
/*															*/
/* closeServer: Close server						    	*/
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	server			: A valid server socket					*/
/************************************************************/
/* Post conditions:											*/
/*	Server is closed									   	*/
/************************************************************/

void closeServer(Tcpsocket server) {
	close(server.fd);
}

/************************************************************/
/*															*/
/* initTcpClient: Create a TCP client					    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/*	host			: Server to call for a connection		*/
/* 	port			: The port to listen on 0<port<65536	*/
/************************************************************/
/* Post conditions:											*/
/*	server.fd > 0: The file descriptor for the connection  	*/
/*	server.fd < 0: A connection could not be established	*/
/************************************************************/

int	initTcpClient(char *host, int port) {
struct	sockaddr_in	sock_addr;
struct	hostent		*host_struct;
int		s;

	if ((host_struct = (struct hostent *) gethostbyname(host)) == NULL)
		return S_HOST_ERR;
	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0)
		return S_SOCK_ERR;
	
	bzero(&sock_addr, sizeof(sock_addr));
	bcopy(host_struct->h_addr, (char *) &sock_addr.sin_addr, host_struct->h_length);
	sock_addr.sin_port = htons((u_short) port);
	sock_addr.sin_family = host_struct->h_addrtype;
	
	if (connect(s, (struct sockaddr *) &sock_addr, sizeof(sock_addr)) < 0)
		return S_PORT_ERR;
	
	return s;
}

/************************************************************/
/*															*/
/* readSocket: Read from the socket						    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	socket			: A valid socket						*/
/*	buff			: Buffer to put result in				*/
/*	bufsize			: Size of buffer						*/
/************************************************************/
/* Post conditions:											*/
/*	>= 0: Number of bytes read							   	*/
/*	< 0 : Error												*/
/************************************************************/

int readSocket(int sock, char *buff, int bufsize) {
int		nbytes = 0, nfds = 0;
fd_set	readfd, exceptfd;
	
	FD_ZERO(&readfd);
	FD_SET(sock, &readfd);
	FD_ZERO(&exceptfd);
	
	if ((nfds = select(sock + 1, &readfd, NULL, NULL, NULL)) < 0)
		return S_WAIT_ERR;
	if (nfds > 0) {
		if ((nbytes = recv(sock, buff, bufsize, 0)) < 0)
			return S_RECV_ERR;
		else
			return nbytes;
	}
	else
		return S_TIME_ERR;
}

/************************************************************/
/*															*/
/* readLSocket: Read a line from socket					    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	sock			: A valid socket						*/
/*	buff			: Buffer to put result in				*/
/*	maxsize			: Size of buffer						*/
/************************************************************/
/* Post conditions:											*/
/*	>= 0: Number of bytes read							   	*/
/*	< 0 : Error												*/
/************************************************************/

int	readLSocket(int sock, char *buff, int maxsize) {
int nfds = 0;
fd_set readfd;
int ret = 0;
		
	while (--maxsize) {
		FD_ZERO(&readfd);
		FD_SET(sock, &readfd);
		ret = 0;
		if ((nfds = select(sock+1, &readfd, NULL, NULL, NULL)) < 0) {
			ret = S_WAIT_ERR;
			break;
		}
		if (recv(sock, buff, 1, 0) != 1) {
			ret = S_RECV_ERR;
			break;
		}
		if (*buff == '\n') {
			ret = 0;
			break;
		}
		if (*buff != '\r' && *buff != 0)
			buff++;
	}
	*buff = 0;
	return ret;
}

/************************************************************/
/*															*/
/* writeSocket: Write to socket							    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	sock			: A valid socket						*/
/*	buff			: Buffer to read from					*/
/*	maxsize			: Size of buffer						*/
/************************************************************/
/* Post conditions:											*/
/*	>= 0: Number of bytes written						   	*/
/*	< 0 : Error												*/
/************************************************************/

int	writeSocket(int sock, char *buff, int size) {
int	nbytes = 0;
int nfds = 0;
fd_set writefd;

	while (1) {
		FD_ZERO(&writefd);
		FD_SET(sock, &writefd);
		if ((nfds = select(sock+1, NULL, &writefd, NULL, NULL)) < 0)
			return S_WAIT_ERR;
		if (nfds) {
			if ((nbytes = send(sock, buff, size, 0)) < 0)
				return S_SEND_ERR;
			buff += nbytes;
			size -= nbytes;
			if (size == 0)
				return 0;
		}
	}
}

/************************************************************/
/*															*/
/* getIP: Get IP from hostname							    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	host			: A valid hostname						*/
/*	ip				: Buffer to hold IP						*/
/************************************************************/
/* Post conditions:											*/
/*	= 0: Found IP										   	*/
/*	< 0: Could not find an IP								*/
/************************************************************/

int	getIP(char *host, char *ip) {
struct hostent *hostptr;
struct in_addr *ptr;
	
	if ((hostptr = (struct hostent *) gethostbyname(host)) == NULL)
		return S_HOST_ERR;
	ptr = (struct in_addr *) *hostptr->h_addr_list;
	strcpy(ip, (char *) inet_ntoa(*ptr));
	return 0;
}

/************************************************************/
/*															*/
/* hostName: Get domain name for host					    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	host		: A valid hostname							*/
/*	ip			: Buffer to hold fully qualified domain name*/
/************************************************************/
/* Post conditions:											*/
/*	= 0: Found fully qualified host name				   	*/
/*	< 0: Could not find a fully qualified host name			*/
/************************************************************/

int	hostName(char *host, char *fqdn) {
struct hostent *hostptr;

	if ((hostptr = (struct hostent *) gethostbyname(host)) == NULL)
		return S_HOST_ERR;
	strcpy(fqdn, hostptr->h_name);
	return 0;
}

/************************************************************/
/*															*/
/* domainName: Get domain name from a dottet IP			    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	host		: Dottet IP									*/
/*	ip			: Buffer to hold fully qualified domain name*/
/************************************************************/
/* Post conditions:											*/
/*	= 0: Found fully qualified host name				   	*/
/*	< 0: Could not find a fully qualified host name			*/
/************************************************************/

int	domainName(char *ip, char *fqdn) {
struct hostent *hostptr;
struct in_addr addr;
	
	addr.s_addr = inet_addr(ip);
	if ((hostptr = (struct hostent *) gethostbyaddr((char *) &addr, 
		sizeof(struct in_addr), AF_INET)) == NULL)
		return S_HOST_ERR;
	strcpy(fqdn, hostptr->h_name);
	return 0;
}

/************************************************************/
/*															*/
/* getPeerIp: Get fully qualified domain name an dottet IP  */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	sock		: A valid socket							*/
/* 	ipfrom		: Buffer to hold IP							*/
/*	ip			: Buffer to hold fully qualified domain name*/
/************************************************************/
/* Post conditions:											*/
/*	None												   	*/
/************************************************************/

void getPeerIp(int sock, char *ipfrom, char *hostfrom) {
struct sockaddr_in from;
size_t foo = sizeof(struct sockaddr_in);
struct hostent *hostptr;

	strcpy(ipfrom, "???.???.???.???");
	strcpy(hostfrom, "?????");
	if (getpeername(sock, (struct sockaddr *) &from, &foo) == 0) {
		strcpy(ipfrom, (char *) inet_ntoa(from.sin_addr));
		hostptr = (struct hostent *) gethostbyaddr((char *) &from.sin_addr,
			sizeof(struct in_addr), AF_INET);
		if (hostptr != NULL)
			strcpy(hostfrom, hostptr->h_name);
	}
}

/************************************************************/
/*															*/
/* noZombie: Avoid leaving zombies						    */
/*														 	*/
/************************************************************/
/* Preconditions:											*/
/* 	sig			: The caught signal							*/
/************************************************************/
/* Post conditions:											*/
/*	None													*/
/************************************************************/

void noZombie(int sig) {
	while (waitpid(-1, NULL, WNOHANG) > 0);
}
